<cfcomponent name="queryCache" output="false" hint="Manages caching of data">
<!--- ------------------------------------------------------------------------------------------------

Component:  queryCache
Author:     Nathan Strutz
Release 1:  8/29/2004

Purpose:    Manage caching of query objects
Notes:      Set instance of this CFC in a persistant scope (session, application, server).
            Query caching is simple...
               Set and get a recordset:
               <cfset myLocalQueryItem = cachedQuery("getThings", "SELECT * FROM table", "myDSN")>
               Erase a cached query so it will be re-cached on the next call to request it:
               <cfset flushQuery("getThings")>

new and cool:  If your database times out or query fails, it sets the timeout to +30 seconds and
			   returns the previously cached recordset.


...         I thought createtimeSpan() + dateTimeObject created a dateTimeObject with the timeSpan
            added to it...?
------------------------------------------------------------------------------------------------ --->

<cfset variables.cachedQueries = structNew()>


<cffunction name="cachedQuery" access="public" output="true" returntype="query" hint="Retrieves and/or sets a cached query">
	<cfargument name="name" type="string" required="true" displayname="variableName" hint="Unique Query Name" />
	<cfargument name="sqlSelect" type="string" required="true" displayname="sqlSelectStatement" hint="Select statement to send to database - any valid query that returns a recordset" />
	<cfargument name="sqlDSN" type="string" required="false" default="#application.dsn#" hint="Your database DSN, Default is ##application.dsn##" />
	<cfargument name="timeout" type="string" required="false" default="" hint="Optional - Timeout period (timespan) for this cached query, use createTimeSpan()" />

	<!--- Temporarily holds new query (if needed). Used in case database times out --->
	<cfset var qry = "">

	<cfif not isNumeric(arguments.timeout) and len(arguments.timeout)>
		<cfthrow message="Timeout parameter is not a valid time span. Use CreateTimeSpan() or leave it blank." />
	</cfif>
	
	<cfif not isDefined("variables.cachedQueries." & arguments.name) or isDate(variables.cachedQueries[arguments.name].timeout) and now() GT variables.cachedQueries[arguments.name].timeout>
		<!--- Query does not yet exist or has timed out --->
		<cftry>
			<cfquery name="qry" datasource="#arguments.sqlDSN#">
				#preserveSingleQuotes(sqlSelect)#
			</cfquery>
			<cfcatch>
				<cfif isDefined("variables.cachedQueries." & arguments.name)>
					<!--- error, probably due to db maintenance, disconnection or timeout --->
					<cfif isDate(variables.cachedQueries[arguments.name].timeout)>
						<!--- if query is on a timeout, reset the timeout to +30 seconds from now --->
						<cfset variables.cachedQueries[arguments.name].timeout = dateAdd("s",30,now()) />
					</cfif>
					<!--- return previously cached query, even though query has error --->
					<cfreturn variables.cachedQueries[arguments.name].query />
				<cfelse>
					<!--- unrecoverable database error --->
					<cfrethrow />
				</cfif>
			</cfcatch>
		</cftry>

		<!--- New resting place for this cached query --->
		<cfset variables.cachedQueries[arguments.name] = structNew() />
		
		<!--- set new query to temp qry variable --->
		<cfset variables.cachedQueries[arguments.name].query = qry />

		<!--- set new timeout period --->
		<cfif len(arguments.timeout)>
			<!--- set new timeout date --->
			<cfset variables.cachedQueries[arguments.name].timeout = parseDateTime(dateFormat(arguments.timeout + now(), "mm/dd/yyyy") & " " & timeFormat(arguments.timeout + now(),"HH:mm:ss tt")) />
		<cfelse>
			<!--- null is never timeout --->
			<cfset variables.cachedQueries[arguments.name].timeout = "" />
		</cfif>



		<cfif not isDefined("variables.cachedQueries." & arguments.name & ".query.recordcount")>
			<cfset flushQuery(arguments.name) />
			<cfthrow message="SQL Statement #arguments.name# did not return a recordset" />
		</cfif>
	</cfif>
	
	<cfreturn variables.cachedQueries[arguments.name].query />
</cffunction>


<cffunction name="flushQuery" access="public" output="false" returntype="void" hint="Flushes a cached query">
	<cfargument name="name" type="string" required="true" hint="Name of query to flush" />
	<cfset structDelete(variables.cachedQueries,arguments.name) />
</cffunction>


<cffunction name="isQueryCached" access="public" output="false" returntype="boolean" hint="Manual query caching control, generally not used - tells wether or not a query is cached">
	<cfargument name="name" type="string" required="true" hint="Name of query to find" />
	<cfreturn isDefined("variables.cachedQueries." & arguments.name) />
</cffunction>

<cffunction name="getCachedQueryKeys" access="public" output="false" returntype="string" hint="Returns a list of all the cached queries">
	<cfreturn structKeyList(variables.cachedQueries) />
</cffunction>


</cfcomponent>